@@ -25,18 +25,31 @@ summariseCodeUse <- function(x,
                              ageGroup = NULL,
                              minCellCount = 5){
 
- codeUse <- getCodeUse(x,
-             cdm = cdm,
-             cohortTable = NULL,
-             cohortId = NULL,
-             timing = "any",
-             countBy = countBy,
-             byConcept = byConcept,
-             byYear = byYear,
-             bySex = bySex,
-             ageGroup = ageGroup,
-             minCellCount = minCellCount) %>%
-   dplyr::mutate(cohort_name = NA)
+  checkmate::assertList(x)
+  if(length(names(x)) != length(x)){
+    cli::cli_abort("Must be a named list")
+  }
+
+  codeUse <- list()
+  for(i in seq_along(x)){
+  cli::cli_inform("Getting use of codes from {names(x)[i]} ({i} of {length(x)})")
+  codeUse[[i]] <- getCodeUse(x[[i]],
+                          cdm = cdm,
+                          cohortTable = NULL,
+                          cohortId = NULL,
+                          timing = "any",
+                          countBy = countBy,
+                          byConcept = byConcept,
+                          byYear = byYear,
+                          bySex = bySex,
+                          ageGroup = ageGroup,
+                          minCellCount = minCellCount) %>%
+      dplyr::mutate(codelist_name = names(x)[i]) %>%
+      dplyr::mutate(cohort_name = NA)
+  }
+  codeUse <- dplyr::bind_rows(codeUse)
+
+
 
  return(codeUse)
 
@@ -78,7 +91,10 @@ summariseCohortCodeUse <- function(x,
                                    ageGroup = NULL,
                                    minCellCount = 5){
 
-
+  checkmate::assertList(x)
+  if(length(names(x)) != length(x)){
+    cli::cli_abort("Must be a named list")
+  }
   checkDbType(cdm = cdm, type = "cdm_reference")
   checkmate::assertTRUE("GeneratedCohortSet" %in% class(cdm[[cohortTable]]))
   checkmate::assertTRUE(all(c("cohort_definition_id", "subject_id", "cohort_start_date",
@@ -91,10 +107,13 @@ summariseCohortCodeUse <- function(x,
 
   cohortCodeUse <- list()
   for(i in seq_along(cohortId)){
+  for(j in seq_along(x)){
   workingCohortName <- CDMConnector::cohort_set(cdm[[cohortTable]]) %>%
     dplyr::filter(.data$cohort_definition_id == cohortId[[i]]) %>%
     dplyr::pull("cohort_name")
-  cohortCodeUse[[i]] <- getCodeUse(x,
+
+  cli::cli_inform(" Getting counts of {names(x)[j]} codes for cohort {workingCohortName}")
+  cohortCodeUse[[paste0(i, "_", j)]] <- getCodeUse(x[[j]],
              cdm = cdm,
              cohortTable = cohortTable,
              cohortId = cohortId[[i]],
@@ -105,8 +124,9 @@ summariseCohortCodeUse <- function(x,
              bySex = bySex,
              ageGroup = ageGroup,
              minCellCount = minCellCount) %>%
+    dplyr::mutate(codelist_name = names(x)[j]) %>%
     dplyr::mutate(cohort_name = workingCohortName)
-  }
+  }}
   cohortCodeUse <- dplyr::bind_rows(cohortCodeUse)
 
   return(cohortCodeUse)
@@ -144,7 +164,6 @@ getCodeUse <- function(x,
 
   checkAgeGroup(ageGroup = ageGroup)
 
-
   if(is.null(attr(cdm, "write_schema"))){
     cli::cli_abort("cdm must have a write_schema specified",
                    call = call)
@@ -154,7 +173,6 @@ getCodeUse <- function(x,
                               tolower(paste0(sample(LETTERS, 4, replace = TRUE),
                                              collapse = "")))
 
-
   codes <- dplyr::tibble(concept_id = x) %>%
     dplyr::left_join(cdm[["concept"]] %>%
                        dplyr::select("concept_id", "domain_id"),
@@ -171,12 +189,8 @@ getCodeUse <- function(x,
                                 timing = timing,
                                 intermediateTable = intermediateTable)
 
-  if(!is.null(records)) {
-    records <- records %>%
-      dplyr::left_join(cdm[["concept"]] %>%
-                         dplyr::select("concept_id", "concept_name"),
-                       by = "concept_id")
-
+  if(!is.null(records) &&
+     (records %>% utils::head(1) %>% dplyr::tally() %>% dplyr::pull("n") > 0)) {
     if(bySex == TRUE | !is.null(ageGroup)){
       records <- records %>%
         PatientProfiles::addDemographics(cdm = cdm,
@@ -206,8 +220,13 @@ getCodeUse <- function(x,
 
     codeCounts <- codeCounts %>%
       dplyr::mutate(group_level = dplyr::if_else(.data$group_name == "By concept",
-                                                 paste0(.data$concept_name, " (",
-                                                        .data$concept_id, ")"),
+                                                 paste0("Standard concept: ",
+                                                   .data$standard_concept_name, " (",
+                                                        .data$standard_concept_id, ")",
+                                                   " Source concept: ",
+                                                   .data$source_concept_name, " (",
+                                                   .data$source_concept_id, ")",
+                                                   " Domain: ", .data$domain_id),
                                                  "Overall")) %>%
       dplyr::mutate(variable_type = "Numeric",
                     variable_level = "Overall",
@@ -218,7 +237,13 @@ getCodeUse <- function(x,
                                     "variable_type",
                                     "estimate_type",
                                     "estimate",
-                                    "estimate_suppressed")))
+                                    "estimate_suppressed",
+                                    "standard_concept_name",
+                                    "standard_concept_id",
+                                    "source_concept_name",
+                                    "source_concept_id",
+                                    "domain_id"
+                                    )))
 
   } else {
     codeCounts <- dplyr::tibble()
@@ -249,19 +274,32 @@ addDomainInfo <- function(codes,
                stringr::str_detect(domain_id,"observation") ~ "observation",
                stringr::str_detect(domain_id,"measurement") ~ "measurement",
                stringr::str_detect(domain_id,"visit") ~ "visit_occurrence",
-               stringr::str_detect(domain_id,"procedure") ~ "procedure_occurrence"
+               stringr::str_detect(domain_id,"procedure") ~ "procedure_occurrence",
+               stringr::str_detect(domain_id,"device") ~ "device_exposure"
              )
     ) %>%
-    dplyr::mutate(concept_id_name =
+    dplyr::mutate(standard_concept_id_name =
                     dplyr::case_when(
                stringr::str_detect(domain_id,"condition") ~ "condition_concept_id",
                stringr::str_detect(domain_id,"drug") ~ "drug_concept_id",
                stringr::str_detect(domain_id,"observation") ~ "observation_concept_id",
                stringr::str_detect(domain_id,"measurement") ~ "measurement_concept_id",
                stringr::str_detect(domain_id,"visit") ~ "visit_concept_id",
-               stringr::str_detect(domain_id,"procedure") ~ "procedure_concept_id"
+               stringr::str_detect(domain_id,"procedure") ~ "procedure_concept_id",
+               stringr::str_detect(domain_id,"device") ~ "device_concept_id"
              )
     ) %>%
+    dplyr::mutate(source_concept_id_name =
+                    dplyr::case_when(
+                      stringr::str_detect(domain_id,"condition") ~ "condition_source_concept_id",
+                      stringr::str_detect(domain_id,"drug") ~ "drug_source_concept_id",
+                      stringr::str_detect(domain_id,"observation") ~ "observation_source_concept_id",
+                      stringr::str_detect(domain_id,"measurement") ~ "measurement_source_concept_id",
+                      stringr::str_detect(domain_id,"visit") ~ "visit_source_concept_id",
+                      stringr::str_detect(domain_id,"procedure") ~ "procedure_source_concept_id",
+                      stringr::str_detect(domain_id,"device") ~ "device_source_concept_id"
+                    )
+    ) %>%
     dplyr::mutate(date_name =
                     dplyr::case_when(
                stringr::str_detect(domain_id,"condition") ~ "condition_start_date",
@@ -269,10 +307,20 @@ addDomainInfo <- function(codes,
                stringr::str_detect(domain_id,"observation") ~ "observation_date",
                stringr::str_detect(domain_id,"measurement") ~ "measurement_date",
                stringr::str_detect(domain_id,"visit") ~ "visit_start_date",
-               stringr::str_detect(domain_id,"procedure") ~ "procedure_date"
+               stringr::str_detect(domain_id,"procedure") ~ "procedure_date",
+               stringr::str_detect(domain_id,"device") ~ "device_exposure_start_date"
              )
     )
 
+  unsupported_domains <- codes %>%
+    dplyr::filter(is.na(.data$table_name)) %>%
+    dplyr::pull("domain_id")
+
+  if(length(unsupported_domains)>0){
+    cli::cli_warn("Concepts included from non-supported domains
+                   ({unsupported_domains})")
+  }
+
   return(codes)
 
 }
@@ -286,7 +334,8 @@ getRelevantRecords <- function(codes,
                                intermediateTable){
 
   tableName <- purrr::discard(unique(codes$table_name), is.na)
-  conceptIdName <- purrr::discard(unique(codes$concept_id_name), is.na)
+  standardConceptIdName <- purrr::discard(unique(codes$standard_concept_id_name), is.na)
+  sourceConceptIdName <- purrr::discard(unique(codes$source_concept_id_name), is.na)
   dateName <- purrr::discard(unique(codes$date_name), is.na)
 
   if(!is.null(cohortTable)){
@@ -319,13 +368,16 @@ if(length(tableName)>0){
   codeRecords <- codeRecords %>%
     dplyr::mutate(date = !!dplyr::sym(dateName[[1]])) %>%
     dplyr::mutate(year = lubridate::year(date)) %>%
-    dplyr::select(dplyr::all_of(c("person_id", conceptIdName[[1]],
+    dplyr::select(dplyr::all_of(c("person_id",
+                                  standardConceptIdName[[1]],
+                                  sourceConceptIdName[[1]],
                            "date", "year"))) %>%
-    dplyr::rename("concept_id" = .env$conceptIdName[[1]]) %>%
+    dplyr::rename("standard_concept_id" = .env$standardConceptIdName[[1]],
+                  "source_concept_id" = .env$sourceConceptIdName[[1]]) %>%
     dplyr::inner_join(codes %>%
                         dplyr::filter(.data$table_name == tableName[[1]]) %>%
-                        dplyr::select("concept_id"),
-                      by = "concept_id",
+                        dplyr::select("concept_id", "domain_id"),
+                      by = c("standard_concept_id"="concept_id"),
                       copy = TRUE) %>%
     CDMConnector::computeQuery(
       name = paste0(intermediateTable,"_grr"),
@@ -335,7 +387,7 @@ if(length(tableName)>0){
     )
 
 } else {
-  codeRecords <- NULL
+  return(NULL)
 }
 
   # get for any additional domains and union
@@ -355,13 +407,16 @@ if(length(tableName)>0){
       workingRecords <-  workingRecords %>%
         dplyr::mutate(date = !!dplyr::sym(dateName[[i+1]])) %>%
         dplyr::mutate(year = lubridate::year(date)) %>%
-        dplyr::select(dplyr::all_of(c("person_id", conceptIdName[[i+1]],
+        dplyr::select(dplyr::all_of(c("person_id",
+                                      standardConceptIdName[[i+1]],
+                                      sourceConceptIdName[[i+1]],
                                       "date", "year"))) %>%
-        dplyr::rename("concept_id" = .env$conceptIdName[[i+1]]) %>%
+        dplyr::rename("standard_concept_id" = .env$standardConceptIdName[[i+1]],
+                      "source_concept_id" = .env$sourceConceptIdName[[i+1]]) %>%
         dplyr::inner_join(codes %>%
                             dplyr::filter(.data$table_name == tableName[[i+1]]) %>%
-                            dplyr::select("concept_id"),
-                          by = "concept_id",
+                            dplyr::select("concept_id", "domain_id"),
+                          by = c("standard_concept_id"="concept_id"),
                           copy = TRUE)
       codeRecords <- codeRecords %>%
         dplyr::union_all(workingRecords)  %>%
@@ -374,6 +429,24 @@ if(length(tableName)>0){
     }
   }
 
+  if(codeRecords %>% utils::head(1) %>% dplyr::tally() %>% dplyr::pull("n") >0){
+  codeRecords <- codeRecords %>%
+    dplyr::left_join(cdm[["concept"]] %>%
+                       dplyr::select("concept_id", "concept_name"),
+                     by = c("standard_concept_id"="concept_id")) %>%
+    dplyr::rename("standard_concept_name"="concept_name") %>%
+    dplyr::left_join(cdm[["concept"]] %>%
+                       dplyr::select("concept_id", "concept_name"),
+                     by = c("source_concept_id"="concept_id")) %>%
+    dplyr::rename("source_concept_name"="concept_name")  %>%
+    CDMConnector::computeQuery(
+      name = paste0(intermediateTable,"_grr_cr"),
+      temporary = FALSE,
+      schema = attr(cdm, "write_schema"),
+      overwrite = TRUE
+    )
+  }
+
   return(codeRecords)
 
 }
@@ -396,7 +469,11 @@ recordSummary <- records %>%
 if(isTRUE(byConcept)) {
   recordSummary <- dplyr::bind_rows(recordSummary,
                    records %>%
-    dplyr::group_by(.data$concept_id, .data$concept_name) %>%
+    dplyr::group_by(.data$standard_concept_id,
+                    .data$standard_concept_name,
+                    .data$source_concept_id,
+                    .data$source_concept_name,
+                    .data$domain_id) %>%
     dplyr::tally(name = "estimate") %>%
     dplyr::mutate(estimate = as.integer(.data$estimate),
                   group_name = "By concept") %>%
@@ -423,9 +500,15 @@ personSummary <- records %>%
 if(isTRUE(byConcept)) {
 personSummary <- dplyr::bind_rows(personSummary,
   records %>%
-    dplyr::select("person_id", "concept_id", "concept_name") %>%
+    dplyr::select("person_id",
+                  "standard_concept_id", "standard_concept_name",
+                  "source_concept_id", "source_concept_name", "domain_id") %>%
     dplyr::distinct() %>%
-    dplyr::group_by(.data$concept_id, .data$concept_name) %>%
+    dplyr::group_by(.data$standard_concept_id,
+                    .data$standard_concept_name,
+                    .data$source_concept_id,
+                    .data$source_concept_name,
+                    .data$domain_id) %>%
     dplyr::tally(name = "estimate") %>%
     dplyr::mutate(estimate = as.integer(.data$estimate),
                   group_name = "By concept") %>%
@@ -531,7 +614,9 @@ groupedCounts <- dplyr::bind_rows(
     dplyr::collect(),
   records %>%
     dplyr::group_by(dplyr::pick(.env$groupBy,
-                                "concept_id", "concept_name")) %>%
+                                "standard_concept_id", "standard_concept_name",
+                                "source_concept_id", "source_concept_name",
+                                "domain_id")) %>%
     dplyr::tally(name = "estimate") %>%
     dplyr::mutate(estimate = as.integer(.data$estimate),
                   group_name = "By concept"
@@ -565,11 +650,15 @@ getGroupedPersonCount <- function(records,
       dplyr::collect(),
     records %>%
       dplyr::select(dplyr::all_of(c("person_id",
-                                    "concept_id", "concept_name",
+                                    "standard_concept_id", "standard_concept_name",
+                                    "source_concept_id", "source_concept_name",
+                                    "domain_id",
                                     .env$groupBy))) %>%
       dplyr::distinct() %>%
       dplyr::group_by(dplyr::pick(.env$groupBy,
-                                  "concept_id", "concept_name")) %>%
+                                  "standard_concept_id", "standard_concept_name",
+                                  "source_concept_id", "source_concept_name",
+                                  "domain_id")) %>%
       dplyr::tally(name = "estimate") %>%
       dplyr::mutate(estimate = as.integer(.data$estimate),
                     group_name = "By concept"
